@@ -16,9 +16,6 @@ use version;
 use Path::Tiny;
 use Fcntl qw/:seek/;
 use Encode;
-use Types::Standard qw< Bool Str Maybe InstanceOf >;
-use Types::Common::Numeric qw< PositiveOrZeroInt >;
-use Type::Utils qw< declare as via coerce from >;
 
 use Moo;
 
@@ -31,11 +28,11 @@ App::Licensecheck - functions for a simple license checker for source files
 
 =head1 VERSION
 
-Version v3.0.4
+Version v3.0.5
 
 =cut
 
-our $VERSION = version->declare("v3.0.4");
+our $VERSION = version->declare("v3.0.5");
 
 =head1 SYNOPSIS
 
@@ -55,6 +52,59 @@ See the script for casual usage.
 
 =cut
 
+my $default_check_regex = q!
+	/[\w-]+$ # executable scripts or README like file
+	|\.( # search for file suffix
+		c(c|pp|xx)? # c and c++
+		|h(h|pp|xx)? # header files for c and c++
+		|S
+		|css|less # HTML css and similar
+		|f(77|90)?
+		|go
+		|groovy
+		|lisp
+		|scala
+		|clj
+		|p(l|m)?6?|t|xs|pod6? # perl5 or perl6
+		|sh
+		|php
+		|py(|x)
+		|rb
+		|java
+		|js
+		|vala
+		|el
+		|sc(i|e)
+		|cs
+		|pas
+		|inc
+		|dtd|xsl
+		|mod
+		|m
+		|md|markdown
+		|tex
+		|mli?
+		|(c|l)?hs
+	)$
+!;
+
+# From dpkg-source
+my $default_ignore_regex = q!
+	# Ignore general backup files
+	~$|
+	# Ignore emacs recovery files
+	(?:^|/)\.#|
+	# Ignore vi swap files
+	(?:^|/)\..*\.swp$|
+	# Ignore baz-style junk files or directories
+	(?:^|/),,.*(?:$|/.*$)|
+	# File-names that should be ignored (never directories)
+	(?:^|/)(?:DEADJOE|\.cvsignore|\.arch-inventory|\.bzrignore|\.gitignore)$|
+	# File or directory names that should be ignored
+	(?:^|/)(?:CVS|RCS|\.pc|\.deps|\{arch\}|\.arch-ids|\.svn|\.hg|_darcs|\.git|
+	\.shelf|_MTN|\.bzr(?:\.backup|tags)?)(?:$|/.*$)
+!;
+
 # also used to cleanup
 my $copyright_indicator_regex = qr!
 	(?:copyright(?:-holders?)? # The full word (or slightly more)
@@ -83,37 +133,105 @@ my $copyright_predisindicator_regex = qr!(
 	^[#]define\s+.*\(c\) # #define foo(c) -- not copyright
 )!ix;
 
-my $EncodingRef = declare as InstanceOf [qw(Encode::Encoding Encode::XS)];
-coerce $EncodingRef, from Str, via { find_encoding($_) };
+has check_regex => (
+	is     => 'rw',
+	lazy   => 1,
+	coerce => sub {
+		my $value = shift;
+		return qr/$default_check_regex/x
+			if $value eq 'common source files';
+		return $value if ref $value eq 'Regexp';
+		return qr/$value/;
+	},
+	default => sub {qr/$default_check_regex/x},
+);
+
+has ignore_regex => (
+	is     => 'rw',
+	lazy   => 1,
+	coerce => sub {
+		my $value = shift;
+		return qr/$default_ignore_regex/x
+			if $value eq 'some backup and VCS files';
+		return $value if ref $value eq 'Regexp';
+		return qr/$value/;
+	},
+	default => sub {qr/$default_ignore_regex/x},
+);
+
+has recursive => (
+	is => 'rw',
+);
 
 has lines => (
 	is      => 'rw',
-	isa     => PositiveOrZeroInt,
 	default => sub {60},
 );
 
 has tail => (
 	is      => 'rw',
-	isa     => PositiveOrZeroInt,
-	default => sub {5000},          # roughly 60 lines of 80 chars
+	default => sub {5000},    # roughly 60 lines of 80 chars
 );
 
 has encoding => (
 	is     => 'rw',
-	isa    => Maybe [$EncodingRef],
-	coerce => $EncodingRef->coercion,
+	coerce => sub {
+		find_encoding( $_[0] ) if ref( $_[0] ) ne 'OBJECT';
+	},
 );
 
 has verbose => (
-	is  => 'rw',
-	isa => Bool,
+	is => 'rw',
+);
+
+has skipped => (
+	is => 'rw',
 );
 
 has deb_fmt => (
-	is  => 'rw',
-	isa => Bool,
+	is => 'rw',
 );
 
+sub find
+{
+	my ( $self, @paths ) = @_;
+
+	my @files       = ();
+	my $paths_count = @paths;
+
+	foreach (@paths) {
+		my $path = path($_);
+
+		if ( $path->is_dir ) {
+			$path->visit(
+				sub {
+					# Silently skip empty files or ignored files
+					return if -z $_ or $_ =~ $self->ignore_regex;
+					if ( $_ =~ $self->check_regex ) {
+						push @files, $_;
+					}
+					else {
+						warn "skipped file $_\n" if $self->skipped;
+					}
+				},
+				{ recurse => $self->recursive }
+			);
+		}
+		elsif ( $path =~ $self->ignore_regex ) {
+
+			# Silently skip ignored files
+			next;
+		}
+		elsif ( $paths_count == 1 or $path =~ $self->check_regex ) {
+			push @files, $path;
+		}
+		else {
+			warn "skipped file $path\n" if $self->skipped;
+		}
+	}
+	return @files;
+}
+
 sub parse
 {
 	my $self = shift;
